/* src/ddmpeg.c
 *
 * Copyright 2003  Ivor Hewitt. All Rights Reserved.
 *
 * This software is distributed under the GNU General Public License,
 * except for the portions which have been provided by VIA which are
 * subject to the copyright notice which can be found in the  file
 * COPYRIGHT.TXT.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *    
 * --------------------------------------------------------------------
 *
 * Enquiries regarding this source can be made directly to:
 *
 * ivor @ ivor.org
 * http://www.ivor.it
 *
 * --------------------------------------------------------------------
 *
 * This source has been derived from the libaries and header files published
 * by VIA for the CLE266 chipset.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/timeb.h>
#include <sys/io.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <pthread.h>

#include "via.h"
#include "via_types.h"
#include "via_privIoctl.h"
#include "ddmpeg.h"
#include "CLE266MPEGSDK.h"

/*
  unsigned long gdwOverlaySupport;
  unsigned long gdwPageFlip;
  unsigned long gdwVideoOn
  gbyMPEG2
  gbSurfaceLost
  dwDisplayingBuffer
  AGPIDX_BUFFERY
  AGPIDX_BUFFERCB
  AGPIDX_BUFFERCR
*/

volatile unsigned char  *lpVidMEMIO;
unsigned char           *lpPrimarySurface;
volatile unsigned char  *lpGMEMIO;
volatile unsigned char  *lpMPEGMMIO;
int                     fVideo;

MPGDEVICE               MPGDevice;
SUBDEVICE               SUBDevice;
LPMPGDEVICE             lpMPGDevice = &MPGDevice;

unsigned char           *gFileBuf;
FILE                    *gFlog;
char                    gBuf[100];
unsigned char           Reg3C4H;
unsigned char           Save_3C4_16;
unsigned char           Save_3C4_17;
unsigned char           Save_3C4_18;

pthread_mutex_t         update_mutex;
pthread_mutex_t         flip_mutex;
int                     flip_mutex_count=0;

static unsigned long        gdwCreateDrvCnt;
static VIAGRAPHICINFO       gVIAGraphicInfo;
static int                  MPEG_ON;
static unsigned long        gdwVideoFlagMPEG;
//static unsigned long        gdwPitch;
//static unsigned long        gdwPageFlip;
static unsigned long        gdwDriverStatus;
//static unsigned long        gdwOffset;
static unsigned long        gdwNewFrame;
static OVERLAYRECORD        overlayRecord;
static unsigned long        gdwPictDesc;
static unsigned long        gSlice_Control_3;

/*long           coded_picture_width;
long           coded_picture_height;
long           chrom_width;
long           chrom_height;
long           coded_picture_widthXheight;
long           chrom_widthXheight;*/
unsigned long  pict_type;
unsigned long  pict_struct;

//CLEX38 change.
static unsigned long Mpeg_Buffer0_Ptr;
static unsigned long Mpeg_Buffer1_Ptr;
static unsigned long Mpeg_Buffer2_Ptr;
static unsigned long Mpeg_Buffer3_Ptr;

static unsigned char FF_flag;

#define VIDEO_REG_NUM  100

typedef struct
{
    unsigned long dwIndex;
    unsigned long dwData;
}VIDEOREGISTER;

static VIDEOREGISTER VidReg[VIDEO_REG_NUM];
static unsigned long gdwVidRegCounter;

unsigned long VIA_MPEGSetFrameBuffer( LPMPGDEVICE lpMPGDevice );
unsigned long VIA_MPEGSliceReceiveData( unsigned long dwByteCount, unsigned char *lpData );
unsigned long VIA_MPEGConnect( LPMPGDEVICE lpMPGDevice );
unsigned long VIA_MPEGDisConnect();
unsigned long VIA_MPEGBeginPicture(LPVIAMPGSURFACE lpMPGSurface );
unsigned long VIA_MPEGDisplayControl( LPVIAMPGSURFACE lpMPGSurface );
unsigned long VIA_MPEGEndPicture();

void WaitHQVFIFOEmpty();
void WaitHQVDone();
void Macro_VidREGFlush();
void Macro_VidREGRec(unsigned long dwAction,unsigned long dwIndex,unsigned long dwData);
void Macro_VidREGFlushFlip();

inline void MpegEnable( unsigned long cmd );

/*
 * MPGOpenDebugFile
 *
 * Status: Complete, Verified.
 */
void MPGOpenDebugFile()
{
    gFileBuf = (unsigned char*)malloc(0x200000);
    gFlog = fopen("/log/MPG-Priv.log","w+");
    setbuffer(gFlog, gFileBuf, 0x200000);
}

/*
 * MPGCloseDebugFile
 *
 * Status: Complete, Verified.
 */
void MPGCloseDebugFile()
{
    if (gFlog)
    {
        fclose(gFlog);
        gFlog=0;
    }
}

/*
 * MPGprintf()
 *
 * Status: Complete, Verified.
 */
void MPGprintf(char *format, ... )
{
    if (gFlog)
    {
        va_list ap;
        strcpy(gBuf," ");

        va_start(ap, format);
        vsprintf(gBuf+strlen(gBuf), format, ap);
        va_end(ap);
    
        fputs(gBuf, gFlog);
        fflush(gFlog);
    }
}

/*
 * Mmap_PrimarySurface
 *
 * Status: Complete, Verified.
 */
unsigned char *Mmap_PrimarySurface()
{
    unsigned char *mem;
    unsigned char *addr;
    unsigned long tmp;
    int fd;

    outb(0x30, 0x3c4);
    tmp = inb(0x3c5);

    addr = (unsigned char*)(tmp << 0x18);

    fd = open("/dev/mem", 2);
    mem = (unsigned char*)mmap(0, 0x1000000, 3, 1, fd, (off_t)addr);
    close(fd);

    return mem;
}

/*
 * Mmap_GraphicEngine
 *
 * Status: Complete, Verified.
 */
unsigned char *Mmap_GraphicEngine()
{
    unsigned char *mem;
    unsigned long tmp;
    unsigned char *addr;
    int fd;

    outb(0x2f, 0x3c4);
    tmp = inb(0x3c5);

    addr = (unsigned char*)(tmp<< 0x18);

    fd = open("/dev/mem",2);
    mem = (unsigned char*)mmap(0,0x1000,3,1,fd,(off_t)addr);
    close(fd);

    return mem;
}


/*
 * Macro_DrvCreated
 *
 * Status: Complete, Verified.
 */
unsigned long Macro_DrvCreated()
{
    if (gdwCreateDrvCnt == 0)
        return 0;
    return 1;
}

/*
 * Macro_CanDestroyDrv
 *
 * Status: Complete, Verified.
 */
unsigned long Macro_CanDestroyDrv()
{
    if (gdwCreateDrvCnt == 0)
        return 1;
    return 0;
}


/*
 * Macro_IncDrvCnt
 *
 * Status: Complete, Verified.
 */
void Macro_IncDrvCnt()
{
    gdwCreateDrvCnt++;
}

/*
 * Macro_DecDrvCnt
 *
 * Status: Complete, Verified.
 */
void Macro_DecDrvCnt()
{
    gdwCreateDrvCnt--;
    if (gdwCreateDrvCnt < 0)
        gdwCreateDrvCnt = 0;
}

/*
 * CreateDriver
 *
 * Status: Complete, Verified.
 */
int CreateDriver()
{
    if (Macro_DrvCreated() != 0)
    {
        Macro_IncDrvCnt();
        return PI_OK;
    }

    Macro_IncDrvCnt();

    fVideo = open("/dev/video2", 2);
    if (fVideo == -1)
    {
        return PI_ERR_CANNOT_OPEN_VIDEO_DEVICE; //3
    }

    if ( ioctl(fVideo, VIA_VID_GET_2D_INFO, &gVIAGraphicInfo) == -1)
    {
        return PI_ERR_CANNOT_USE_IOCTL; //4
    }

    memset(lpMPGDevice, 0, sizeof(MPGDEVICE));

    overlayRecord.dwMPEGDeinterlaceMode = 0;

    ioperm(0, 0x400, 1);
    iopl(3); 

    Reg3C4H = inb(0x3c4);

    lpPrimarySurface = Mmap_PrimarySurface();
    lpGMEMIO = Mmap_GraphicEngine();

    lpVidMEMIO = lpGMEMIO+0x200;
    lpMPEGMMIO = lpGMEMIO+0xc00;

/*  No longer in CLEXF40038
    outb(0x16, 0x3c4);
    Save_3C4_16 = inb(0x3c5);
    outb(0x17, 0x3c4);
    Save_3C4_17 = inb(0x3c5);
    outb(0x18, 0x3c4);
    Save_3C4_18 = inb(0x3c5);
*/
    pthread_mutex_init(&update_mutex, 0);
    pthread_mutex_init(&flip_mutex, 0);

    return PI_OK;
}

/*
 * DestroyDriver
 *
 * Status: Complete, Verified.
 */
int DestroyDriver()
{
    Macro_DecDrvCnt();

    if (!Macro_CanDestroyDrv())
    {
        return PI_OK;
    }

    close(fVideo);
    ioperm(0, 0x400, 1);

    iopl(3);

    outb(Reg3C4H, 0x3c4);

    munmap( (void *)lpPrimarySurface, 0x1000000);
    munmap( (void *)lpGMEMIO, 0x1000);

    pthread_mutex_destroy(&update_mutex); //CHECK

    MPEG_ON = 0;
    return PI_OK;
}

void InitSurfaceManager()
{
    gVIAGraphicInfo.Screen[0].TotalVRAM = 0x2000000;
    gVIAGraphicInfo.Screen[0].VideoHeapBase = 0x340000;
    gVIAGraphicInfo.Screen[0].VideoHeapEnd = 0x1fbdfff;
    gVIAGraphicInfo.Screen[0].dwWidth = 1024;
    gVIAGraphicInfo.Screen[0].dwHeight= 768;
    gVIAGraphicInfo.Screen[0].dwBPP = 16;
    gVIAGraphicInfo.Screen[0].dwPitch = 2048;
    gVIAGraphicInfo.Screen[0].dwRefreshRate = 60;

    ioctl(fVideo, 0x410076c1, &gVIAGraphicInfo);
}

void FreeSurfaceManager()
{
    ioctl(fVideo, 0x76c2, &gVIAGraphicInfo);
}

/*
 * DispatchVGARevisionID 
 *
 * Status: Complete
 */
unsigned long DispatchVGARevisionID()
{
    if (gVIAGraphicInfo.RevisionID > 15)
        return 16;
    else
        return gVIAGraphicInfo.RevisionID;
}


/*
 * CreateSurface
 *
 * Status: Complete, Verified
 * Checked -HRM
 */
unsigned long CreateSurface( LPDDSURFACEDESC lpDDSurfaceDesc ) 
{
    DDLOCK lock;

    if (lpDDSurfaceDesc == 0)
        return PI_ERR;

//    vfLoadColorSetting();
    
    switch ( lpDDSurfaceDesc->dwFourCC)
    {

    case FOURCC_YV12:
    case FOURCC_YUY2:
        ioctl( fVideo, 0x402876C7, lpDDSurfaceDesc );
        break;
        
        case FOURCC_VIA:
        {

            unsigned long c = DispatchVGARevisionID();

            gdwVideoFlagMPEG = VIDEO_SHOW |
                VIDEO_MPEG_INUSE |
                VIDEO_HQV_INUSE |
                VIDEO_1_INUSE |
                MPEG_USE_V1 |
                ((c == 0x10) ? MPEG_USE_HW_FLIP : 0 ) |
                MPEG_USE_HQV; //0x8D030000;
        

            ioctl(fVideo, VIA_VID_CREATESURFACE, lpDDSurfaceDesc );

            lock.dwFourCC = FOURCC_VIA; //0x4E4B4C57; //CHECK

            ioctl(fVideo, VIA_VID_LOCKSURFACE, &lock );

            memcpy(&MPGDevice, &lock.MPGDev,  sizeof(MPGDEVICE));

            if (VIA_MPEGConnect(lpMPGDevice)!=0)
                return PI_ERR_CANNOT_CREATE_SURFACE;

            VIA_MPEGSetFrameBuffer( &MPGDevice );
        }
        break;

        case FOURCC_SUBP:
            ioctl( fVideo, VIA_VID_CREATESURFACE, lpDDSurfaceDesc );

            lock.dwFourCC = FOURCC_SUBP; //0x50425553; //CHECK

            ioctl( fVideo, VIA_VID_LOCKSURFACE, &lock );

            memcpy( &MPGDevice, &lock.MPGDev, sizeof(MPGDEVICE));
            memcpy( &SUBDevice, &lock.SubDev, sizeof(SUBDEVICE));
            break;
    }

    return PI_OK;
}

/*
 * DestroySurface
 *
 * Status: Complete, Verified
 * Checked - HRM
 */

unsigned long DestroySurface(LPDDSURFACEDESC lpDDSurfaceDesc)
{
    if (lpDDSurfaceDesc == 0)
        return PI_ERR;

    switch(lpDDSurfaceDesc->dwFourCC)
    {
        case FOURCC_VIA:
            VIA_MPEGDisConnect();
            ioctl(fVideo, VIA_VID_DESTROYSURFACE, lpDDSurfaceDesc );

            gdwVideoFlagMPEG = 0;
            break;
    
        case FOURCC_SUBP:
            ioctl(fVideo, VIA_VID_DESTROYSURFACE, lpDDSurfaceDesc );
            break;
        case FOURCC_YV12:
        case FOURCC_YUY2:
            break;

    }

    return PI_OK;
}

/*
 * VIABeginPicture
 * 
 * Status: Complete, Verified.
 */
unsigned long VIABeginPicture(void   *lpMPGSurface)
{
    pthread_mutex_lock(&flip_mutex);
    flip_mutex_count++;
    pthread_mutex_unlock(&flip_mutex);

    VIA_MPEGBeginPicture( lpMPGSurface );
    overlayRecord.dwMpegDecoded = 1;

    pthread_mutex_lock(&flip_mutex);
    flip_mutex_count--;
    pthread_mutex_unlock(&flip_mutex);

    return PI_OK;
}

/*
 * VIAEndPicture
 *
 * Status: Complete, Verified.
 */
unsigned long VIAEndPicture(void *lpMPGSurface)
{
    VIA_MPEGEndPicture(lpMPGSurface);
    return PI_OK;
}

/*
 * VIASliceReceiveData
 * 
 * Status: Complete, Verified.
 */
unsigned long VIASliceReceiveData(unsigned long dwByteCount,unsigned char *lpData)
{
    pthread_mutex_lock(&flip_mutex);
    flip_mutex_count++;
    pthread_mutex_unlock(&flip_mutex);

    VIA_MPEGSliceReceiveData( dwByteCount, lpData );

    pthread_mutex_lock(&flip_mutex);
    flip_mutex_count--;
    pthread_mutex_unlock(&flip_mutex);

    return PI_OK;
}


/*
 * VIADisplayControl
 *
 * Status: Complete, Verified.
 * Although why is devType not used.
 */
unsigned long VIADisplayControl(unsigned long devType, void *lpData )
{

    unsigned long mode = VIDInD(V_COMPOSE_MODE);
    mode &= 0x3FFFFFFF;

    LPVIAMPGSURFACE lpSurface = lpData;

    int var_10 = 0;

    unsigned long flag = gdwVideoFlagMPEG;
    if (lpSurface->dwTaskType == 2) //Display ?
    {
        Macro_VidREGRec(0,0,0);

        if ((flag & MPEG_USE_HW_FLIP)  == 0)
        {
            if (flag & VIDEO_HQV_INUSE)
            {
                Macro_VidREGRec(1, HQV_SRC_STARTADDR_Y, lpMPGDevice->dwMPEGYPhysicalAddr[lpSurface->dwDisplayBuffIndex]); //0x70
                Macro_VidREGRec(1, HQV_SRC_STARTADDR_U, lpMPGDevice->dwMPEGCbPhysicalAddr[lpSurface->dwDisplayBuffIndex]);
                Macro_VidREGRec(1, HQV_SRC_STARTADDR_V, lpMPGDevice->dwMPEGCrPhysicalAddr[lpSurface->dwDisplayBuffIndex]);
                
                overlayRecord.dwMPEGProgressiveMode = lpSurface->dwMPEGProgressiveMode; //140
                overlayRecord.dwDisplayPictStruct = lpSurface->dwDisplayPictStruct;
                overlayRecord.dwDisplayBuffIndex = lpSurface->dwDisplayBuffIndex;
                
                unsigned long control;
                
                if (lpSurface->dwDisplayPictStruct == 1)
                {
                    overlayRecord.dwMPEGDeinterlaceMode = 1;
                    if (overlayRecord.dwMPEGProgressiveMode == 0)
                    {
                        control = VIDInD(HQV_CONTROL) | 
                            HQV_FIELD_UV|
                            HQV_DEINTERLACE|
                            HQV_FIELD_2_FRAME|
                            HQV_FRAME_2_FIELD|
                            HQV_SW_FLIP|
                            HQV_FLIP_ODD|
                            HQV_FLIP_STATUS; //0x170031;
                    }
                    else
                    {
                        control = VIDInD(HQV_CONTROL) |
                            HQV_DEINTERLACE|
                            HQV_FIELD_2_FRAME|
                            HQV_FRAME_2_FIELD|
                            HQV_SW_FLIP|
                            HQV_FLIP_ODD|
                            HQV_FLIP_STATUS; // 0x70031;
                    }
                }
                else if (lpSurface->dwDisplayPictStruct == 2 )
                {
                    overlayRecord.dwMPEGDeinterlaceMode = 1;
                    if (overlayRecord.dwMPEGProgressiveMode == 0)
                    {
                        control = (VIDInD(HQV_CONTROL) & 0x0FFFFFFDF)|
                            HQV_FIELD_UV|
                            HQV_DEINTERLACE|
                            HQV_FIELD_2_FRAME|
                            HQV_FRAME_2_FIELD|
                            HQV_SW_FLIP|
                            HQV_FLIP_STATUS; //0x170011;
                    }
                    else
                    {
                        control = (VIDInD(HQV_CONTROL) & 0x0FFFFFFDF)|
                            HQV_DEINTERLACE|
                            HQV_FIELD_2_FRAME|
                            HQV_FRAME_2_FIELD|
                            HQV_SW_FLIP|
                            HQV_FLIP_STATUS; //0x70011;
                    }
                }
                else
                {
                    overlayRecord.dwMPEGDeinterlaceMode = 0;
                    control = (VIDInD(HQV_CONTROL) & 0x0FFFFFFDF) |
                        HQV_SW_FLIP|
                        HQV_FLIP_STATUS; // 0x11;
                }
                
                Macro_VidREGRec(1, HQV_CONTROL, control);
                WaitHQVDone();
                Macro_VidREGFlushFlip();           
            }
            else
            {
                overlayRecord.dwDisplayPictStruct = lpSurface->dwDisplayPictStruct;
                if (overlayRecord.dwMPEGDeinterlaceMode == 1)
                {
                    if (lpSurface->dwDisplayPictStruct == 3)
                        lpSurface->dwDisplayPictStruct = 1;
                }
                
                Macro_VidREGRec(1, V1_STARTADDR_Y0,
                                lpMPGDevice->dwMPEGYPhysicalAddr[lpSurface->dwDisplayBuffIndex]+overlayRecord.dwOffset);
                Macro_VidREGRec(1, V1_STARTADDR_CB0,
                                lpMPGDevice->dwMPEGCbPhysicalAddr[lpSurface->dwDisplayBuffIndex]+overlayRecord.dwUVoffset);
                Macro_VidREGRec(1, V1_STARTADDR_CR0,
                                lpMPGDevice->dwMPEGCrPhysicalAddr[lpSurface->dwDisplayBuffIndex]+overlayRecord.dwUVoffset);
                
                mode |= 0x80000000;
                Macro_VidREGRec(1, V_COMPOSE_MODE, mode);
                Macro_VidREGFlush();
            }
        }
        else
        {
            while (flip_mutex_count == 0)
            {
                overlayRecord.dwDisplayPictStruct = lpSurface->dwDisplayPictStruct;
                lpMPGDevice->dwMPEGDisplayIndex = lpSurface->dwDisplayBuffIndex;

                if (overlayRecord.dwMPEGDeinterlaceMode == 1 &&
                    overlayRecord.dwDisplayPictStruct == 3)
                    lpSurface->dwDisplayPictStruct = 1; //6c

                if (FF_flag)        
            WaitHQVFIFOEmpty();

                VIA_MPEGDisplayControl( lpSurface );

                pthread_mutex_lock(&flip_mutex);
                flip_mutex_count++;
                pthread_mutex_unlock(&flip_mutex);

                var_10=1;
            }
        }       
    }     

    pthread_mutex_lock(&flip_mutex);
    if (var_10 != 0)
        flip_mutex_count--;

    pthread_mutex_unlock(&flip_mutex);
    return 0;
}

/*
 * MPEGFlipCurrentFrame
 *
 * Status: Complete, Verified
 */
void MPEGFlipCurrentFrame()
{
    unsigned long cl = VIDInD(0xc00);

    if ((overlayRecord.dwDisplayPictStruct == 3) &&
        (overlayRecord.dwMPEGDeinterlaceMode == 0))
    {
        VIDOutD(0xc00, lpMPGDevice->dwMPEGDisplayIndex | 0x8C);
    }
    else
    {
        VIDOutD(0xc00, lpMPGDevice->dwMPEGDisplayIndex | 0x84);
    }

    if ( !(cl & 0x80) ) //CHECK
    {
        VIDOutD(0xc00,cl);
    }
}

/*
 * VIASubPicture
 *
 * Status: Complete, Verified.
 */
unsigned long VIASUBPicture(void *lpData)
{
    unsigned long flag = gdwVideoFlagMPEG;
    LPVIASUBPICT lpSubp = lpData;

    Macro_VidREGRec(0, 0, 0);

    unsigned long stride=0;
    switch (lpSubp->dwTaskType)
    {
        int i;
        case 6:
            for (i=0; i <= 0x0f; i++)
            {
                long table = (lpSubp->dwRamTab[i] << 8) | (i << 4) | 7; 
                Macro_VidREGRec( 1, RAM_TABLE_CONTROL, table );
            }
            Macro_VidREGFlush();
            return 0;
            break;

        case 2:
            overlayRecord.dwSPstartAddr = SUBDevice.dwSUBPhysicalAddr[lpSubp->dwDisplayBuffIndex];
            
            if ((flag & 0x4000000) == 0)
                return 0;
            Macro_VidREGRec(1, SUBP_STARTADDR, overlayRecord.dwSPstartAddr);
            stride = VIDInD(SUBP_CONTROL_STRIDE) | 0x10000;            
            break;
        
        case 8:
            lpMPGDevice->gdwSUBP_NotVisible = 1;
            if ((flag & 0x4000000) == 0)
                return 0;
            stride = VIDInD(SUBP_CONTROL_STRIDE) & 0x0FFFEFFFF;
            break;
    }

    Macro_VidREGRec( 1, SUBP_CONTROL_STRIDE, stride );
    if ((flag & MPEG_USE_HW_FLIP) != 0)
    {
        Macro_VidREGFlush();
        MPEGFlipCurrentFrame();
        return 0;
    }

    Macro_VidREGFlush();
    return 0;
}


/*
 * VIASetDeInterlaceMode
 *
 * Status: Complete, Verified.
 */
unsigned long VIASetDeInterlaceMode(unsigned long dwMode)
{
    overlayRecord.dwMPEGDeinterlaceMode = dwMode;
    return 0;
}

/*
 * VIADriverProc
 *
 * Status: Complete, Verified. 
 * See CHECK though.
 */
unsigned long VIADriverProc(unsigned long wAction,void *lpParam)
{
    long ret = 1;
    
    switch( wAction )
    {
        case 0:
            ret = CreateDriver();
            break;
        case 1:
            ret = DestroyDriver();
            break;
        case 2:
            ret = CreateSurface(lpParam);
            break;
        case 3:
            ret = DestroySurface(lpParam);
            break;
        case 4:
            if (ioctl(fVideo,VIA_VID_LOCKSURFACE, (LPDDLOCK)lpParam) == -1)
                return 1;
            
            switch (((LPDDLOCK)lpParam)->dwFourCC)
            {
                case FOURCC_YUY2:
                case FOURCC_YV12: {
                    LPSWDEVICE d = &((LPDDLOCK)lpParam)->SWDevice;
                    d->lpSWOverlaySurface[0] = d->dwSWPhysicalAddr[0] + lpPrimarySurface;
                    d->lpSWOverlaySurface[1] = d->dwSWPhysicalAddr[1] + lpPrimarySurface;
                }   break;
                
                case FOURCC_SUBP: {
                    LPSUBDEVICE d = &((LPDDLOCK)lpParam)->SubDev;
                    d->lpSUBOverlaySurface[0] = d->dwSUBPhysicalAddr[0] + lpPrimarySurface;
                    d->lpSUBOverlaySurface[1] = d->dwSUBPhysicalAddr[1] + lpPrimarySurface;
                }   break;
                
                case FOURCC_VIA: {
                    LPMPGDEVICE d = &((LPDDLOCK)lpParam)->MPGDev;
                    int i;
                    for (i=0; i<NUM_FRAME_BUFFERS; i++)
                        d->lpMPEGOverlaySurface[i] = d->dwMPEGPhysicalAddr[i] + lpPrimarySurface;
                }   break;
                
                case FOURCC_ALPHA: {
                    LPALPHADEVICE d = &((LPDDLOCK)lpParam)->Alphadev;
                    d->lpALPOverlaySurface = d->dwALPPhysicalAddr + lpPrimarySurface;
                }   break;
            }
            
            ret = 0;
            break;
        case 6:
            if (ioctl(fVideo,VIA_VID_UPDATEOVERLAY, (LPDDUPDATEOVERLAY)lpParam) == -1)
                return 1;
            ret = 0;
            MPEG_ON = 1;
            break;
            
        case 0x12:
            FF_flag=1;
            break;
            
        case 0x13:
            FF_flag=0;
            break;
            
        case 0x14:
            InitSurfaceManager();
            break;
            
        case 0x15:
            FreeSurfaceManager();
            break;
    }
    
    return ret;
}

/*
 * PrivPutImage
 *
 * Status: Complete
 * but odd.
 *
 */
unsigned long *InParam;
int PrivPutImage(unsigned char *buf)
{
    InParam = (unsigned long*)buf;
    unsigned long command = *(unsigned long*)InParam++;
    unsigned long wParam = *(unsigned long*)InParam++;

    switch ( command )
    {
        case DRIVERPROC: //0xc: //38d5
            VIADriverProc(wParam, //wAction
                          InParam);
            break;
        case BEGINPICTURE: //0x9:
            VIABeginPicture(InParam);
            break;
        case ENDPICTURE: //0xa:
            VIAEndPicture(InParam);
            break;
        case SLICERECEIVEDATA: //0xb:
        {
            unsigned long count = *(unsigned long*)InParam++;
            VIASliceReceiveData( count, //dwByteCount
                                 (unsigned char*)InParam); //lpData
        }
        break;
        case DISPLAYCONTROL: //0xd:
            VIADisplayControl(wParam, //devType
                              InParam); //lpData
            break;
        case SUBPICTURE: //0xe:
            VIASUBPicture(InParam); //lpSubp
            break;
        case SETDEINTERLACEMODE: //0xf:
            VIASetDeInterlaceMode(wParam); //dwMode
            break;
        default:
            break;
    };
    return 0;
}


/*
 * SetIOPermissions
 *
 * Status: Complete, Verified
 */
void SetIOPermissions()
{
    ioperm(0, 0x400, 1);
    iopl(3);
}

/*
 * EnableMMIO
 *
 * Status: Complete, Verified
 */
void EnableMMIO()
{
    unsigned long tmp;

    outb( 0x1a, 0x3c4);
    tmp = inb( 0x3c5);
    tmp |= 0x68;
    outb( tmp, 0x3c5);
}


/*
 * VIA_MPEGSetFrameBuffer
 *
 * Status: Complete, Verified.
 */
unsigned long VIA_MPEGSetFrameBuffer( LPMPGDEVICE pMPGDevice )
{
    lpMPGDevice = pMPGDevice;

    if (lpMPGDevice->byDeviceType == 0)
    {
        MPGOutD(0x20, lpMPGDevice->dwMPEGYPhysicalAddr[0] >> 3);
        MPGOutD(0x24, lpMPGDevice->dwMPEGCbPhysicalAddr[0] >> 3);
        MPGOutD(0x28, lpMPGDevice->dwMPEGCrPhysicalAddr[0] >> 3);

        MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[1] >> 3);
        MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[1] >> 3);
        MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[1] >> 3);

        MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[2] >> 3);
        MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[2] >> 3);
        MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[2] >> 3);

        MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[3] >> 3);
        MPGOutD(0x48, lpMPGDevice->dwMPEGCbPhysicalAddr[3] >> 3);
        MPGOutD(0x4c, lpMPGDevice->dwMPEGCrPhysicalAddr[3] >> 3);

        VIDOutD(0x54, lpMPGDevice->dwMPEGYPhysicalAddr[0]);
        VIDOutD(0x8c, lpMPGDevice->dwMPEGCbPhysicalAddr[0]);
        VIDOutD(0xf0, lpMPGDevice->dwMPEGCrPhysicalAddr[0]);

        VIDOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[1]);
        VIDOutD(0xe4, lpMPGDevice->dwMPEGCbPhysicalAddr[1]);
        VIDOutD(0xf4, lpMPGDevice->dwMPEGCrPhysicalAddr[1]);

        VIDOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[2]);
        VIDOutD(0xe8, lpMPGDevice->dwMPEGCbPhysicalAddr[2]);
        VIDOutD(0xf8, lpMPGDevice->dwMPEGCrPhysicalAddr[2]);

        VIDOutD(0x5c, lpMPGDevice->dwMPEGYPhysicalAddr[3]);
        VIDOutD(0xec, lpMPGDevice->dwMPEGCbPhysicalAddr[3]);
        VIDOutD(0xfc, lpMPGDevice->dwMPEGCrPhysicalAddr[3]);
    }
    return 0;
}

/*
 * VIA_MPEGSliceReceiveData
 *
 * Status: Complete, Verified.
 */
unsigned long VIA_MPEGSliceReceiveData( unsigned long dwByteCount, unsigned char *lpData )
{
    unsigned long *data = (unsigned long*)lpData;

    int i;
    for (i=1; i<= dwByteCount; i++)
    {
        unsigned long val = *data;
	
	MPGOutD(0x9c, val);

        int j;
        for (j = 0; j < val; j += 4)
        {
            data++;
	    MPGOutD(0xa0, *data);
        }
        data++;
    }

    return 0;
}

/*
 * VIA_MPEGConnect
 *
 * Status: Complete, Verified.
 */
unsigned long VIA_MPEGConnect( LPMPGDEVICE pMPGDevice )
{
    lpMPGDevice = pMPGDevice;

/*    coded_picture_width = lpMPGDevice->dwWidth;
    coded_picture_height = lpMPGDevice->dwHeight;

    chrom_width = coded_picture_width >> 1;
    chrom_height= coded_picture_height >> 1;

    coded_picture_widthXheight = coded_picture_width * coded_picture_height;
    chrom_widthXheight = chrom_width * chrom_height;

    gdwPitch = lpMPGDevice->dwPitch;
    gdwPageFlip = 0;
*/
    
    if ( lpMPGDevice->dwDeviceID == 0x3022 || lpMPGDevice->dwDeviceID == 0x3122) //CHECK
    {
        MPGOutD(0x98, 0x400000);
   
        int i;
        for (i=0;i<6;i++)
            MpegEnable(0x43);

        if (lpMPGDevice->dwEnableErrorConcealment != 0x0)
        {
            MpegEnable(0xc3);
        }

        MPGOutD(0x50,(lpMPGDevice->dwPitch >> 3) | ((lpMPGDevice->dwPitch >> 4) << 0x10));
    gdwDriverStatus = 4;        
    }
    else
    {
        gdwDriverStatus = 1;
        return 1;
    }
       
    return 0;
}

/*
 * VIA_MPEGDisConnect
 *
 * Status: Complete, Verified.
 */
unsigned long VIA_MPEGDisConnect()
{
    if ( gdwDriverStatus == 1)
        return 1;

    unsigned long tmp = lpMPGDevice->dwDeviceID;
    if (!( tmp == 0x3022 || tmp == 0x3122) )
    {
        gdwDriverStatus = 1;
        return 1;
    }

    MPGOutD(0x0c, MPGInD(0x0c) & 0x0FFFFFFBC);
    return 0;
}


/*
 * VIA_MPEGBeginPicture
 *
 * Status: Complete, Verified.
 * Except the ftime code is slightly different, CHECK.
 */

unsigned long VIA_MPEGBeginPicture( LPVIAMPGSURFACE lpMPGSurface )
{
    unsigned long oldframe = lpMPGSurface->dwOldRefFrame;
    unsigned long newframe = lpMPGSurface->dwRefFrame;
    pict_type = lpMPGSurface->dwPictureType;
    pict_struct = lpMPGSurface->dwDecodePictStruct;
    gdwNewFrame = lpMPGSurface->dwDecodeBuffIndex;
    unsigned long dwAlternateScan = lpMPGSurface->dwAlternateScan;

    if (lpMPGDevice->dwEnableErrorConcealment)
    {
    int i;
    for (i=0; i<0x14;i++)
    {
        MPGOutD(0x08, 0);
    }

     MPGOutD(0x98, 0x400000);

        for (i=0;i<6;i++)
            MpegEnable(0x43);

        MpegEnable(0xc3);
    }

    //TODO - this is a CLEX38 change, needs the new structures to work.
    // CHECK Haven't had time to check the offsets yet.

   if (gVIAGraphicInfo.SetMpeg_FBNumber == 0)
   {
       MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[gdwNewFrame])>>3);
       MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[gdwNewFrame])>>3);
       MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[gdwNewFrame])>>3);
   } 
   else
   {//4db2
       if (gdwNewFrame == 0)
       {
           if (Mpeg_Buffer0_Ptr == 0)
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[4])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[4])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[4])>>3);
               Mpeg_Buffer0_Ptr = 1;
           }
           else
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[gdwNewFrame])>>3);
               Mpeg_Buffer0_Ptr = 0;
           }
       }
       else if (gdwNewFrame == 1)
       {
           if (Mpeg_Buffer1_Ptr == 0)
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[5])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[5])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[5])>>3);
               Mpeg_Buffer1_Ptr = 1;
           }
           else
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[gdwNewFrame])>>3);
               Mpeg_Buffer1_Ptr = 0;
           }
       }
       else if (gdwNewFrame == 2)
       {
           if (Mpeg_Buffer2_Ptr == 0)
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[6])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[6])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[6])>>3);
               Mpeg_Buffer2_Ptr = 1;
           }
           else
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[gdwNewFrame])>>3);
               Mpeg_Buffer2_Ptr = 0;
           }
       }
       else
       {
           if (Mpeg_Buffer3_Ptr == 0)
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[4])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[4])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[4])>>3);
               Mpeg_Buffer3_Ptr = 1;
           }
           else
           {
               MPGOutD(0x20, (lpMPGDevice->dwMPEGYPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x24, (lpMPGDevice->dwMPEGCbPhysicalAddr[gdwNewFrame])>>3);
               MPGOutD(0x28, (lpMPGDevice->dwMPEGCrPhysicalAddr[gdwNewFrame])>>3);
               Mpeg_Buffer3_Ptr = 0;
           }
       }
   }
   //5097
   
//2
    
    gdwPictDesc = (pict_type<<4)|(pict_struct<<2)|(dwAlternateScan << 6);
    MPGOutD(0x0, gdwPictDesc);
    
//3
    unsigned int i;
    for (i=0; i<=1; i++)
    {
        MPGOutD(0x5c, i);
        
        unsigned int j;
        for (j=0; j <= 0x0f; j++)
        {
            MPGOutD(0x60, lpMPGSurface->dwQMatrix[i][j]);
        }
    }
    
    unsigned long tmp;
    tmp = (lpMPGSurface->dwMBwidth << 0x12) & 0x03FC0000;
    
    if ( lpMPGSurface->dwMpeg2 )
        tmp |= 0x10000;
    
    if ( lpMPGSurface->dwTopFirst )
        tmp |= 0x8000;
    
    if ( lpMPGSurface->dwFramePredDct )
        tmp |= 0x4000;
    
    tmp |= lpMPGSurface->dwMBAmax & 0x3fff;

    MPGOutD(0x90, tmp);
    
    tmp = 0x100000;
    tmp = 1+ tmp / lpMPGSurface->dwMBwidth;
    tmp = (tmp << 4) & 0x0FFFFF0;
    
    if ( lpMPGSurface->dwIntravlc )
        tmp |= 0x1000000;
    
    tmp |= (lpMPGSurface->dwDcPrec << 2) & 0x0c;

    if (lpMPGSurface->dwQscaleType)
        tmp |= 2;
    
    if (lpMPGSurface->dwConcealMV)
        tmp |= 1;
    
    MPGOutD(0x94, tmp);
    
    tmp =  0x0A00000;

    // TODO this is a CLEX38 change needs new structures to work
    //
    // CHECK Haven't had time to check the offsets yet.
    if (gVIAGraphicInfo.SetMpeg_FBNumber == 0)
    {
        MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[oldframe]>>3);
        MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[oldframe]>>3);
        MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[oldframe]>>3);
        
        MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[newframe]>>3);
        MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[newframe]>>3);
        MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[newframe]>>3);
    }
    else
    {
    //5203
        if (oldframe == 0)
        {
            if (Mpeg_Buffer0_Ptr == 0)
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[0]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[0]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[0]>>3);
            }
            else
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[oldframe+4]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[oldframe+4]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[oldframe+4]>>3);
            }
        }
        else if (oldframe == 1)
        {
            if (Mpeg_Buffer1_Ptr == 0)
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[1]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[1]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[1]>>3);
            }
            else
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[oldframe+5]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[oldframe+5]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[oldframe+5]>>3);
            }
        }
        else if (oldframe == 2)
        {
            if (Mpeg_Buffer2_Ptr == 0)
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[2]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[2]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[2]>>3);
            }
            else
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[oldframe+6]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[oldframe+6]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[oldframe+6]>>3);
            }
        }
        else
        {
            if (Mpeg_Buffer3_Ptr == 0)
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[0]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[0]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[0]>>3);
            }
            else
            {
                MPGOutD(0x2c, lpMPGDevice->dwMPEGYPhysicalAddr[oldframe+4]>>3);
                MPGOutD(0x30, lpMPGDevice->dwMPEGCbPhysicalAddr[oldframe+4]>>3);
                MPGOutD(0x34, lpMPGDevice->dwMPEGCrPhysicalAddr[oldframe+4]>>3);
            }
        }


        if (newframe == 0)
        {
            if (Mpeg_Buffer0_Ptr == 0)
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[0]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[0]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[0]>>3);
            }
            else
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[newframe+4]>>3);
            }
        }
        else if (newframe == 1)
        {
            if (Mpeg_Buffer1_Ptr == 0)
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[1]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[1]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[1]>>3);
            }
            else
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[newframe+4]>>3);
            }
        }
        else if (newframe == 2)
        {
            if (Mpeg_Buffer2_Ptr == 0)
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[2]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[2]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[2]>>3);
            }
            else
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[newframe+4]>>3);
            }
        }
        else
        {
            if (Mpeg_Buffer3_Ptr == 0)
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[0]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[0]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[0]>>3);
            }
            else
            {
                MPGOutD(0x38, lpMPGDevice->dwMPEGYPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x3c, lpMPGDevice->dwMPEGCbPhysicalAddr[newframe+4]>>3);
                MPGOutD(0x40, lpMPGDevice->dwMPEGCrPhysicalAddr[newframe+4]>>3);
            }
        }
       
        //>>538b
    }//5493    
    
        
    if (lpMPGSurface->dwSecondField)
        tmp |= 0x100000;
    
    tmp |= 0x60000;
    
    gSlice_Control_3 =
        ((lpMPGSurface->FHMVRange & 0xf) | tmp)|
        ((lpMPGSurface->FVMVRange << 0x4) & 0xf0) |
        ((lpMPGSurface->BHMVRange << 0x8) & 0xf00) |
        ((lpMPGSurface->BVMVRange << 0xc) & 0xf000);

    MPGOutD(0x98,gSlice_Control_3);
    
    return 0;
}

/*
 * VIA_MPEGEndPicture
 *
 * Status: Complete
 */ 
    
unsigned long VIA_MPEGEndPicture()
{
    return 0;
}

/*
 * VIA_MPEGDisplayControl
 *
 * Status: Complete, Verified
 */
unsigned long VIA_MPEGDisplayControl( LPVIAMPGSURFACE lpMPGSurface )
{
    //TODO this is a CLEX38 change needs new structures to work.
    //
    // CHECK Haven't had time to check the offsets yet.

    if (gVIAGraphicInfo.SetMpeg_FBNumber == 0)
    {//5611
        MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
        MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
        MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
    }
    else
    {
        if (lpMPGSurface->dwDisplayBuffIndex  == 0)
        {
            if (Mpeg_Buffer0_Ptr == 0)
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
            else
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
        }
        else if (lpMPGSurface->dwDisplayBuffIndex  == 1)
        {
            if (Mpeg_Buffer1_Ptr == 0)
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
            else
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
        }
        else if (lpMPGSurface->dwDisplayBuffIndex  == 2)
        {
            if (Mpeg_Buffer2_Ptr == 0)
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
            else
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
        }
        else
        {
            if (Mpeg_Buffer3_Ptr == 0)
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
            else
            {
                MPGOutD(0x44, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x48, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
                MPGOutD(0x4c, lpMPGDevice->dwMPEGYPhysicalAddr[4+lpMPGSurface->dwDisplayBuffIndex]>>3);
            }
        }
    }
    
    if ( lpMPGDevice->dwDeviceID == 0x3022 || lpMPGDevice->dwDeviceID == 0x3122) 
    {
        if (DispatchVGARevisionID() == 0x10)
        {
        MPGOutD(0, 0x3080 | gdwPictDesc | (lpMPGSurface->dwDisplayPictStruct<<0x10));
        }
        else
        {
        MPGOutD(0,lpMPGSurface->dwDisplayPictStruct*4|lpMPGSurface->dwDisplayBuffIndex|0x80);
        MPGOutD(0,gdwPictDesc);
        }
    }
    
    return 0;
}

/*
 * WaitForMPEG
 *
 * Status: 
 */

void WaitForMPEG ()
{
    struct timeb time;
    unsigned short now;
    unsigned short then;
    
    ftime(&time);
    then = time.millitm;
//CHECK
    if ((MPGInD(0x54) & 0x27f) != 0x204)
    {
        do 
        {
            ftime(&time);
            now = time.millitm;
            now -= then;
            now += 997;
        }
        while ( now <= 998 || (MPGInD(0x54) & 0x27f) != 0x204);
    }    
}

void ResetMpegEngine()
{
    int i;
    for (i=0; i<14; i++)
    {
        MPGOutD(0x8, 0);
    }
    
    MPGOutD(0x98, 0x400000);

    for (i=0; i<6; i++)
        MpegEnable(0x43);

    MpegEnable(0xc3);
}

inline void MpegEnable( unsigned long cmd )
{
    MPGOutD(0x0c,cmd);
    MPGOutD(0x10,0);
    MPGOutD(0x14,0);
    MPGOutD(0x18,0);
    MPGOutD(0x1c,0);
}

/*
 * Verified.
 */
void MpegDisable( unsigned long arg_0 )
{
    MPGOutD(0x0c, MPGInD(0x0c) & (arg_0 ^= 0xffffffff));
}

/*
 * Verified.
 */
void WaitHQVIdle()
{
    while (! VIDInD(HQV_CONTROL) & HQV_IDLE);
}

/*
 * Verified.
 */
void WaitVideoCommandFire()
{
    while ((VIDInD(V_COMPOSE_MODE) & V1_COMMAND_FIRE) ||
        (VIDInD(V_COMPOSE_MODE) & V3_COMMAND_FIRE));
}

/*
 * Verified.
 */
void WaitHQVFlip()
{
    while (!VIDInD(HQV_CONTROL) & 1);
}

/*
 * Verified.
 * This creates slightly different code, CHECK
 */
void WaitHQVFlipClear(unsigned long dwData)
{
    VIDOutD(HQV_CONTROL, dwData);

    while ( VIDInD(HQV_CONTROL) & 1)
    {
    VIDOutD(HQV_CONTROL, VIDInD(HQV_CONTROL) | 1);
    }
}

/*
 * Verified.
 */
void WaitHQVFIFOEmpty()
{
    while (VIDInD(HQV_CONTROL) & V1_FIFO_THRESHOLD16 );
}

void WaitVBI(void)
{
    while (VIDInD(0) & VBI_STATUS);
}

void WaitHQVDone()
{
    while (VIDInD(HQV_CONTROL) & 0x10)
    {
        usleep(1);
    }    
}

void Macro_VidREGFlush()
{
    while ( (VIDInD(V_COMPOSE_MODE) & V1_COMMAND_FIRE) ||
        (VIDInD(V_COMPOSE_MODE) & V3_COMMAND_FIRE));

    int i;
    for (i=0; i < gdwVidRegCounter; i++)
    {
        VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
    }
}

void Macro_VidREGFlushFlip()
{
    while ( (VIDInD(V_COMPOSE_MODE) & V1_COMMAND_FIRE) || (VIDInD(V_COMPOSE_MODE) & V3_COMMAND_FIRE));

    int i;
    for (i=0; i < gdwVidRegCounter; i++)
    {
        VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
    }
    
}

void Macro_VidREGRec(unsigned long dwAction,unsigned long dwIndex,unsigned long dwData)
{
    switch (dwAction)
    {
    case 0:
            gdwVidRegCounter = 0;
            break;
    case 1:
            VidReg[gdwVidRegCounter].dwIndex = dwIndex;
            VidReg[gdwVidRegCounter].dwData = dwData;
            gdwVidRegCounter++;
            break;
    }
}

/*
 * Macro_VidREGFlushVPE
 *
 * Status: Complete
 */
void Macro_VidREGFlushVPE(void)
{
    while ( (VIDInD(V_COMPOSE_MODE) & V1_COMMAND_FIRE) || (VIDInD(V_COMPOSE_MODE) & V3_COMMAND_FIRE));

    int i;
    for (i=0; i<gdwVidRegCounter; i++)
    {
        VIDOutD(VidReg[i].dwIndex, VidReg[i].dwData);
    }
}

/*
 * Macro_VidREGRecVPE
 *
 * Status: Complete
 */
void Macro_VidREGRecVPE(unsigned long dwAction,unsigned long dwIndex,unsigned long dwData)
{
    switch (dwAction)
    {
        case 0:
            gdwVidRegCounter = 0;
            break;
        case 1:
            VidReg[gdwVidRegCounter].dwIndex = dwIndex;
            VidReg[gdwVidRegCounter].dwData = dwData;
            gdwVidRegCounter++;
            break;
    }

}

void vfLoadColorSetting()
{
    //TODO
}

